#include <stdio.h>
#include <srts/stratego.h>
#include <stratego-lib/stratego-lib.h>
#include <readline/readline.h>
#include <readline/history.h>
#include "aterm-readline.h"

ATerm GNU_Readline_readline(ATerm prompt_term) {
  const char* prompt = AT_getString(prompt_term);
  ATerm result_term;

  char* result = readline(prompt);
  if(result == NULL) {
    _fail(prompt_term);
  }

  result_term = ATmakeString(result);
  free(result);
  return result_term;
}

ATerm GNU_Readline_set_instream(ATerm tstream) {
  FILE* stream;
  stream = stream_from_term(tstream);

  rl_instream = stream;
  return tstream;
}


ATerm GNU_Readline_get_instream(void) {
  return stream_to_term(rl_instream);
}

ATerm GNU_Readline_set_outstream(ATerm tstream) {
  FILE* stream;
  stream = stream_from_term(tstream);

  rl_outstream = stream;
  return tstream;
}

ATerm GNU_Readline_get_outstream(void) {
  return stream_to_term(rl_outstream);
}

ATerm GNU_Readline_add_history(ATerm command_term) {
  const char* command = AT_getString(command_term);
  add_history(command);
  return command_term;
}

ATerm GNU_Readline_read_history(ATerm file_term) {
  const char* file = AT_getString(file_term);
  int result = read_history(file);
  return (ATerm) ATmakeInt(result);
}

ATerm GNU_Readline_write_history(ATerm file_term) {
  const char* file = AT_getString(file_term);
  int result = write_history(file);
  return (ATerm) ATmakeInt(result);
}

char* compentry_helper(const char* text, int state);
static StrCL rl_completion_entry_strat;

ATerm GNU_Readline_set_completion_entry_strat(StrCL generator) {
  rl_bind_key ('\t', rl_complete);
  rl_completion_entry_function = compentry_helper;
  rl_completion_entry_strat = generator;
  return (ATerm) ATempty;
}

char* strduppp(const char *s) {
  char* p;

  p = (char*) malloc(strlen(s) + 1);
  if(p != NULL) {
    strcpy(p, s);
  }

  return p;
}

char* compentry_helper(const char* text, int state) {
  static ATermList all;
  char* result;

  if (!state) {
    char* mytext = strduppp(text);
    all = (ATermList) cl_fun(rl_completion_entry_strat)(NULL, ATmakeString(mytext));
    free(mytext);
  }

  if(all == NULL || ATisEmpty(all)) {
    result = NULL;
  } else {
    ATerm next = ATgetFirst(all);
    all = ATgetNext(all);

    /* ATfprintf(stderr, "[info] GNU Readline completion: %t \n", next); */

    if(!ATisString(next)) {
      ATfprintf(stderr, "[error] GNU Readline completion: %t \n", next);
      result = NULL;
    } else {
      result = strduppp(AT_getString(next));
    }
  }

  return result;
}





